"""
Generate a colorscheme using imagemagick.
"""
import os
import re
import shutil
import subprocess
import sys

from .settings import CACHE_DIR, COLOR_COUNT
from . import util


def imagemagick(color_count, img):
    """Call Imagemagick to generate a scheme."""
    if shutil.which("magick"):
        magick_command = ["magick", "convert"]

    elif shutil.which("convert"):
        magick_command = ["convert"]

    else:
        print("error: imagemagick not found, exiting...\n"
              "error: wal requires imagemagick to function.")
        sys.exit(1)

    colors = subprocess.run([*magick_command, img,
                             "-resize", "25%", "+dither",
                             "-colors", str(color_count),
                             "-unique-colors", "txt:-"],
                            stdout=subprocess.PIPE)

    return colors.stdout.splitlines()


def gen_colors(img, color_count):
    """Format the output from imagemagick into a list
       of hex colors."""
    raw_colors = imagemagick(color_count, img)

    index = 0
    while len(raw_colors) - 1 < color_count:
        index += 1
        raw_colors = imagemagick(color_count + index, img)

        print("colors: Imagemagick couldn't generate a", color_count,
              "color palette, trying a larger palette size",
              color_count + index)

        if index > 20:
            print("colors: Imagemagick couldn't generate a suitable scheme",
                  "for the image. Exiting...")
            sys.exit(1)

    # Remove the first element because it isn't a color code.
    del raw_colors[0]

    return [re.search("#.{6}", str(col)).group(0) for col in raw_colors]


def sort_colors(img, colors):
    """Sort the generated colors and store them in a dict that
       we will later save in json format."""
    raw_colors = colors[:1] + colors[8:] + colors[8:-1]

    # Darken the background color slightly.
    if raw_colors[0][1] != "0":
        raw_colors[0] = util.darken_color(raw_colors[0], 0.25)

    # Manually adjust colors.
    raw_colors[7] = util.lighten_color(raw_colors[7], 0.25)
    raw_colors[8] = util.darken_color(raw_colors[7], 0.30)
    raw_colors[15] = util.lighten_color(raw_colors[15], 0.25)

    colors = {}
    colors["wallpaper"] = img
    colors["special"] = {}
    colors["colors"] = {}

    colors["special"]["background"] = raw_colors[0]
    colors["special"]["foreground"] = raw_colors[15]
    colors["special"]["cursor"] = raw_colors[15]

    for index, color in enumerate(raw_colors):
        colors["colors"]["color%s" % index] = color

    return colors


def get(img, cache_dir=CACHE_DIR,
        color_count=COLOR_COUNT, notify=False):
    """Get the colorscheme."""
    # _home_dylan_img_jpg.json
    cache_file = img.replace("/", "_").replace("\\", "_").replace(".", "_")
    cache_file = os.path.join(cache_dir, "schemes", cache_file + ".json")

    if os.path.isfile(cache_file):
        colors = util.read_file_json(cache_file)
        print("colors: Found cached colorscheme.")

    else:
        util.msg("wal: Generating a colorscheme...", notify)

        colors = gen_colors(img, color_count)
        colors = sort_colors(img, colors)

        util.save_file_json(colors, cache_file)
        util.msg("wal: Generation complete.", notify)

    return colors


def file(input_file):
    """Import colorscheme from json file."""
    data = util.read_file_json(input_file)

    if "wallpaper" not in data:
        data["wallpaper"] = "None"

    return data
